@@ -1,6 +1,7 @@
 
 from collections.abc import MutableMapping
 from .registry import get_filesystem_class
+from .core import split_protocol
 
 
 class FSMap(MutableMapping):
@@ -35,7 +36,8 @@ class FSMap(MutableMapping):
         self.fs = fs
         self.root = fs._strip_protocol(root).rstrip('/')  # we join on '/' in _key_to_str
         if create:
-            self.fs.mkdir(root)
+            if not self.fs.exists(root):
+                self.fs.mkdir(root)
         if check:
             if not self.fs.exists(root):
                 raise ValueError("Path %s does not exist. Create "
@@ -50,7 +52,7 @@ class FSMap(MutableMapping):
         try:
             self.fs.rm(self.root, True)
             self.fs.mkdir(self.root)
-        except (IOError, OSError):
+        except:
             pass
 
     def _key_to_str(self, key):
@@ -70,15 +72,24 @@ class FSMap(MutableMapping):
         key = self._key_to_str(key)
         try:
             result = self.fs.cat(key)
-        except (IOError, OSError):
+        except:
             if default is not None:
                 return default
             raise KeyError(key)
         return result
 
+    def pop(self, key, default=None):
+        result = self.__getitem__(key, default)
+        try:
+            del self[key]
+        except KeyError:
+            pass
+        return result
+
     def __setitem__(self, key, value):
         """Store value in key"""
         key = self._key_to_str(key)
+        self.fs.mkdirs(self.fs._parent(key), exist_ok=True)
         with self.fs.open(key, 'wb') as f:
             f.write(value)
 
@@ -90,9 +101,15 @@ class FSMap(MutableMapping):
     def __iter__(self):
         return self.keys()
 
+    def __len__(self):
+        return len(self.keys())
+
     def __delitem__(self, key):
         """Remove key"""
-        self.fs.rm(self._key_to_str(key))
+        try:
+            self.fs.rm(self._key_to_str(key))
+        except:
+            raise KeyError
 
     def __contains__(self, key):
         """Does key exist in mapping?"""
@@ -135,8 +152,8 @@ def get_mapper(url, check=False, create=False, **kwargs):
     -------
     ``FSMap`` instance, the dict-like key-value store.
     """
-    protocol = url.split(':', 1)[0]
+    protocol, path = split_protocol(url)
     cls = get_filesystem_class(protocol)
     fs = cls(**kwargs)
     # Removing protocol here - could defer to each open() on the backend
-    return FSMap(url, fs, check, create)
+    return FSMap(path, fs, check, create)
